# zope modules
from zope.testing.doctest import NORMALIZE_WHITESPACE, ELLIPSIS
from zope.configuration.xmlconfig import XMLConfig
from zope.testing.doctestunit import DocFileSuite
from zope.app.testing import placelesssetup
from zope.publisher.browser import TestRequest
import transaction, zope.app

# zc modules
import zc.resourcelibrary

# jz modules
from jz.testing.setup import placefulSetUp
import jz.magic, jz.common

# python modules
import unittest


def setUp(test):
	"""Set Up Zope 3 test environment."""

	placelesssetup.setUp()

	XMLConfig("configure.zcml", zope.app)()
	XMLConfig("meta.zcml", zc.resourcelibrary)()
	XMLConfig("configure.zcml", jz.common)()
	XMLConfig("meta.zcml", jz.magic)()
	XMLConfig("configure.zcml", jz.magic)()

	test.globs['TestRequest'] = TestRequest


def test_suite():
	return unittest.TestSuite((
		DocFileSuite('../readme.txt', setUp=setUp, optionflags=NORMALIZE_WHITESPACE|ELLIPSIS),
	))


if __name__ == '__main__':
	unittest.main(defaultTest='test_suite')


r"""Sample data for tests."""

# zope modules
from zope.interface import implements
from zope.annotation.interfaces import IAttributeAnnotatable

# jz modules
from jz.stream.interfaces import IStream
from jz.filerepresentation.interfaces import IFileName


class StreamContent(object):
	"""Stream read from file."""

	implements(IStream, IFileName, IAttributeAnnotatable)

	@property
	def data(self):
		return "I am a sample stream of content"

	name = "content"


# zope modules
from zope.app.file.interfaces import IFile
from zope.interface import Interface
from zope.schema import Bytes, TextLine
from zope.i18nmessageid import MessageFactory


_ = MessageFactory("jz.stream")


class IStream(Interface):
	"""Interface for a generic data stream."""

	data = Bytes(
		title=_("Data"),
		)


class IStreamView(Interface):
	"""Interface for a stream view."""

	data = Bytes(
		title=_("Data"),
		)

	filename = TextLine(
		title=_("File name"),
		)

	mimetype = TextLine(
		title=_("Mime type"),
		)


# zope modules
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty

# custom modules
from interfaces import IStream


class Stream(object):
	__doc__ = IStream.__doc__

	implements(IStream)

	data = FieldProperty(IStream['data'])


from setuptools import setup, find_packages

setup(
	name = "jz.stream",
	version = "0.1",
	description = """Generic data stream classes and views.""",
	author_email = "gabi@jbox-comp.com",
	author = "Gabi Shaar",
	license = "GPL",
	url = "http://www.jellofishi.com/zope",
	packages = find_packages(),
	namespace_packages = ["jz",],
	package_data = {'': ['*.txt', '*.zcml']},
	install_requires = [
		'setuptools',
		'jz.common',
		'jz.magic',
		'jz.filerepresentation',
		'jz.datetime',
	],
	)



# zope modules
from zope.interface import implements
from zope.publisher.browser import BrowserView
from zope.component import getUtility
from zope.dublincore.interfaces import IZopeDublinCore
from zope.app.publisher.browser.fileresource import setCacheControl
from zope.datetime import time as timeFromDateTimeString

# jz modules
from jz.magic.interfaces import IFileMagic
from jz.filerepresentation.interfaces import IFileName
from interfaces import IStream, IStreamView
from jz.datetime import timestamp
from jz.common import removeAllAdapters

# python modules
import tempfile, os, datetime

# twisted modules
from twisted.web2.http_headers import generateDateTime


class StreamView(BrowserView):
	"""Provides a generic browser view for a non-html data stream.

	In the default implementation, information about the stream is gathered from adapters and utilities	applied to the object.

	"""

	implements(IStreamView)

	def __init__(self, *args, **kargs):
		BrowserView.__init__(self, *args, **kargs)

	@property
	def mimetype(self):
		"""Return mime type.

		Default implementation uses jz.magic utility.
		"""

		return getUtility(IFileMagic).getMimeType(self.data)

	@property
	def filename(self):
		"""Return file name.

		Default implementation uses context's name and mime-type."""

		basename = IFileName(self.context).name
		extension = getUtility(IFileMagic).getFileExtension(self.data)
		return "%s.%s" % (basename, extension)

	@property
	def disposition(self):
		"""Content disposition (rfc2183).

		Possible values: inline, attachment.
		"""

		return "inline"

	@property
	def data(self):
		"""Return data stream."""

		return IStream(self.context).data

	def __call__(self):
		response = self.request.response
		request = self.request

		# Content:
		response.setHeader('Content-Disposition','%s; filename=%s' % (self.disposition, self.filename))
		response.setHeader('Content-Type', self.mimetype)
		response.setHeader('Content-Length', len(self.data))

		# Modification time:
		dublincore = IZopeDublinCore(removeAllAdapters(self.context))
		last_mod = timestamp(dublincore.modified)

		# Copied from fileresource.py
		header = request.getHeader('If-Modified-Since', None)
		if header is not None:
			header = header.split(';')[0]
			try:
				mod_since=long(timeFromDateTimeString(header))
			except:
				mod_since=None
			#print "MOD: since %s mod %s" % (mod_since, last_mod)
			if mod_since is not None:
				if last_mod > 0 and last_mod <= mod_since:
					response.setStatus(304)
					return ''

		response.setHeader('Last-Modified', generateDateTime(last_mod))

		# Cache (defaults to 86400 seconds/1 day):
		setCacheControl(response)

		# Content data:
		return self.data

		#NOTE: other way of sending data. return a file handle
		#tf = tempfile.TemporaryFile()
		#tf.write(self.data)
		#NOTE: flush() here fixes problem of file not being downloaded completely.
		#tf.flush()
		#return tf